@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,13 +26,12 @@
 #include "render_handler.h"
 
 #include "gnash.h"
-#include "types.h"
-#include "image.h"
+#include "RGBA.h"
+#include "GnashImage.h"
 #include "utility.h"
 #include "log.h"
 
-#include "types.h"
-#include "image.h"
+#include "GnashImage.h"
 #include "utility.h"
 #include "Range2d.h"
 
@@ -193,11 +192,11 @@ private:
 class oglScopeMatrix : public boost::noncopyable
 {
 public:
-  oglScopeMatrix(const matrix& m)
+  oglScopeMatrix(const SWFMatrix& m)
   {
     glPushMatrix();
 
-    // Multiply (AKA "append") the new matrix with the current OpenGL one.
+    // Multiply (AKA "append") the new SWFMatrix with the current OpenGL one.
     float mat[16];
     memset(&mat[0], 0, sizeof(mat));
     mat[0] = m.sx / 65536.0f;
@@ -236,7 +235,27 @@ point middle(const point& a, const point& b)
   return point(0.5 * (a.x + b.x), 0.5 * (a.y + b.y));
 }
 
+namespace {
 
+class
+PointSerializer
+{
+public:
+    PointSerializer(std::vector<boost::int16_t>& dest)
+        :
+        _dest(dest)
+    {}
+    
+    void operator()(const point& p)
+    {
+        _dest.push_back(p.x);
+        _dest.push_back(p.y);
+    }
+private:
+    std::vector<boost::int16_t>& _dest;
+};
+
+}
 
 // Unfortunately, we can't use OpenGL as-is to interpolate the curve for us. It
 // is legal for Flash coordinates to be outside of the viewport, which will
@@ -413,7 +432,7 @@ bool isEven(const size_t& n)
 
 // Use the image class copy constructor; it's not important any more
 // what kind of image it is.
-bitmap_info_ogl::bitmap_info_ogl(image::ImageBase* image, GLenum pixelformat,
+bitmap_info_ogl::bitmap_info_ogl(GnashImage* image, GLenum pixelformat,
                                  bool ogl_accessible)
 :
   _img(image->clone()),
@@ -505,8 +524,8 @@ bitmap_info_ogl::upload(boost::uint8_t* data, size_t width, size_t height)
 }
 
 void
-bitmap_info_ogl::apply(const gnash::matrix& bitmap_matrix,
-                       render_handler::bitmap_wrap_mode wrap_mode)
+bitmap_info_ogl::apply(const gnash::SWFMatrix& bitmap_matrix,
+                       bitmap_wrap_mode wrap_mode)
 {
   glEnable(_ogl_img_type);
 
@@ -527,7 +546,7 @@ bitmap_info_ogl::apply(const gnash::matrix& bitmap_matrix,
   
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
   
-  if (wrap_mode == render_handler::WRAP_CLAMP) {  
+  if (wrap_mode == WRAP_CLAMP) {  
     glTexParameteri(_ogl_img_type, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
     glTexParameteri(_ogl_img_type, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
   } else {
@@ -535,12 +554,12 @@ bitmap_info_ogl::apply(const gnash::matrix& bitmap_matrix,
     glTexParameteri(_ogl_img_type, GL_TEXTURE_WRAP_T, GL_REPEAT);
   }
     
-  // Set up the bitmap matrix for texgen.
+  // Set up the bitmap SWFMatrix for texgen.
     
   float inv_width = 1.0f / _orig_width;
   float inv_height = 1.0f / _orig_height;
     
-  const gnash::matrix& m = bitmap_matrix;
+  const gnash::SWFMatrix& m = bitmap_matrix;
   glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
   float p[4] = { 0, 0, 0, 0 };
   p[0] = m.sx / 65536.0f * inv_width;
@@ -598,7 +617,7 @@ public:
 
     // Flip the image, since (0,0) by default in OpenGL is the bottom left.
     gluOrtho2D(-oversize, oversize, oversize, -oversize);
-    // Restore the matrix mode to the default.
+    // Restore the SWFMatrix mode to the default.
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
 
@@ -642,14 +661,17 @@ public:
 #endif
   }    
 
-  virtual bitmap_info*  create_bitmap_info_rgb(image::ImageRGB* im)
+  virtual BitmapInfo* createBitmapInfo(std::auto_ptr<GnashImage> im)
   {
-    return new bitmap_info_ogl(im, GL_RGB, ogl_accessible());
-  }
-
-  virtual bitmap_info*  create_bitmap_info_rgba(image::ImageRGBA* im)
-  {
-    return new bitmap_info_ogl(im, GL_RGBA, ogl_accessible());
+      switch (im->type())
+      {
+          case GNASH_IMAGE_RGB:
+              return new bitmap_info_ogl(im.get(), GL_RGB, ogl_accessible());
+          case GNASH_IMAGE_RGBA:
+                return new bitmap_info_ogl(im.get(), GL_RGBA, ogl_accessible());
+          default:
+                std::abort();
+      }
   }
 
   // Since we store drawing operations in display lists, we take special care
@@ -657,7 +679,8 @@ public:
   // anti-aliased with the rest of the drawing. Since display lists cannot be
   // concatenated this means we'll add up with several display lists for normal
   // drawing operations.
-  virtual void drawVideoFrame(image::ImageBase* frame, const matrix* m, const rect* bounds)
+  virtual void drawVideoFrame(GnashImage* frame, const SWFMatrix* m,
+          const rect* bounds, bool /*smooth*/)
   {
     GLint index;
 
@@ -688,7 +711,7 @@ public:
     _render_indices.push_back(index);
   }
   
-  virtual void reallyDrawVideoFrame(image::ImageBase* frame, const matrix* m, const rect* bounds)
+  virtual void reallyDrawVideoFrame(GnashImage* frame, const SWFMatrix* m, const rect* bounds)
   {
   
     if (frame->type() == GNASH_IMAGE_RGBA)
@@ -735,7 +758,7 @@ public:
 
     glPopAttrib();
 
-    // Restore the default matrix mode.
+    // Restore the default SWFMatrix mode.
     glMatrixMode(GL_MODELVIEW); 
     
   }
@@ -863,26 +886,32 @@ public:
 
     glFlush(); // Make OpenGL execute all commands in the buffer.
   }
-    
+  
   /// Draw a line-strip directly, using a thin, solid line. 
   //
   /// Can be used to draw empty boxes and cursors.
-  virtual void
-  draw_line_strip(const boost::int16_t* coords, int vertex_count, const rgba& color,
-                  const matrix& mat)
+  virtual void drawLine(const std::vector<point>& coords, const rgba& color,
+                  const SWFMatrix& mat)
   {
     oglScopeMatrix scope_mat(mat);
 
+    const size_t numPoints = coords.size();
+
     glColor3ub(color.m_r, color.m_g, color.m_b);
 
+    std::vector<boost::int16_t> pointList;
+    pointList.reserve(numPoints * 2);
+    std::for_each(coords.begin(), coords.end(), PointSerializer(pointList));
+
     // Send the line-strip to OpenGL
     glEnableClientState(GL_VERTEX_ARRAY);
-    glVertexPointer(2, GL_SHORT, 0 /* tight packing */, coords);
-    glDrawArrays(GL_LINE_STRIP, 0, vertex_count);
+    glVertexPointer(2, GL_SHORT, 0 /* tight packing */, &pointList.front());
+    glDrawArrays(GL_LINE_STRIP, 0, numPoints);
 
     // Draw a dot on the beginning and end coordinates to round lines.
-    //   glVertexPointer: skip all but the first and last coordinates in the line.
-    glVertexPointer(2, GL_SHORT, (sizeof(boost::int16_t) * 2) * (vertex_count - 1), coords);
+    // glVertexPointer: skip all but the first and last coordinates in the line.
+    glVertexPointer(2, GL_SHORT, (sizeof(boost::int16_t) * 2) *
+            (numPoints - 1), &pointList.front());
     glEnable(GL_POINT_SMOOTH); // Draw a round (antialiased) point.
     glDrawArrays(GL_POINTS, 0, 2);
     glDisable(GL_POINT_SMOOTH);
@@ -894,7 +923,7 @@ public:
   // NOTE: this implementation can't handle complex polygons (such as concave
   // polygons.
   virtual void  draw_poly(const point* corners, size_t corner_count, 
-    const rgba& fill, const rgba& outline, const matrix& mat, bool masked)
+    const rgba& fill, const rgba& outline, const SWFMatrix& mat, bool masked)
   {
     if (corner_count < 1) {
       return;
@@ -989,7 +1018,7 @@ public:
     
     std::vector<line_style> dummy_ls;
     
-    draw_subshape(path_vec, matrix(), dummy_cx, dummy_fs, dummy_ls);
+    draw_subshape(path_vec, SWFMatrix(), dummy_cx, dummy_fs, dummy_ls);
   }
   
   virtual void disable_mask()
@@ -1185,7 +1214,7 @@ public:
     }    
   }
 
-  void apply_fill_style(const fill_style& style, const matrix& mat, const cxform& cx)
+  void apply_fill_style(const fill_style& style, const SWFMatrix& mat, const cxform& cx)
   {
       int fill_type = style.get_type();
       
@@ -1202,9 +1231,9 @@ public:
         {
                     
           bitmap_info_ogl* binfo = static_cast<bitmap_info_ogl*>(style.need_gradient_bitmap());       
-          matrix m = style.get_gradient_matrix();
+          SWFMatrix m = style.getGradientMatrix();
           
-          binfo->apply(m, render_handler::WRAP_CLAMP); 
+          binfo->apply(m, bitmap_info_ogl::WRAP_CLAMP); 
           
           break;
         }        
@@ -1213,7 +1242,7 @@ public:
         {
           bitmap_info_ogl* binfo = static_cast<bitmap_info_ogl*>(style.get_bitmap_info());
 
-          binfo->apply(style.get_bitmap_matrix(), render_handler::WRAP_REPEAT);
+          binfo->apply(style.getBitmapMatrix(), bitmap_info_ogl::WRAP_REPEAT);
           break;
         }
                 
@@ -1225,7 +1254,7 @@ public:
           
           assert(binfo);
 
-          binfo->apply(style.get_bitmap_matrix(), render_handler::WRAP_CLAMP);
+          binfo->apply(style.getBitmapMatrix(), bitmap_info_ogl::WRAP_CLAMP);
           
           break;
         } 
@@ -1242,7 +1271,7 @@ public:
   
   
   
-  bool apply_line_style(const line_style& style, const cxform& cx, const matrix& mat)
+  bool apply_line_style(const line_style& style, const cxform& cx, const SWFMatrix& mat)
   {
   //  GNASH_REPORT_FUNCTION;
      
@@ -1342,7 +1371,7 @@ public:
   }
     
   void
-  draw_outlines(const PathVec& path_vec, const PathPointMap& pathpoints, const matrix& mat,
+  draw_outlines(const PathVec& path_vec, const PathPointMap& pathpoints, const SWFMatrix& mat,
                 const cxform& cx, const std::vector<fill_style>& fill_styles,
                 const std::vector<line_style>& line_styles)
   {
@@ -1493,9 +1522,9 @@ public:
     return subshapes;
   }
   
-  /// Takes a path and translates it using the given matrix.
+  /// Takes a path and translates it using the given SWFMatrix.
   void
-  apply_matrix_to_paths(std::vector<path>& paths, const matrix& mat)
+  apply_matrix_to_paths(std::vector<path>& paths, const SWFMatrix& mat)
   {  
     std::for_each(paths.begin(), paths.end(),
                   boost::bind(&path::transform, _1, boost::ref(mat)));
@@ -1505,7 +1534,7 @@ public:
 
   void
   draw_subshape(const PathVec& path_vec,
-    const matrix& mat,
+    const SWFMatrix& mat,
     const cxform& cx,
     const std::vector<fill_style>& fill_styles,
     const std::vector<line_style>& line_styles)
@@ -1579,11 +1608,8 @@ public:
 // 5. Profit!
 
   virtual void
-  draw_shape_character(shape_character_def *def, 
-    const matrix& mat,
-    const cxform& cx,
-    const std::vector<fill_style>& fill_styles,
-    const std::vector<line_style>& line_styles)
+  draw_shape_character(shape_character_def *def, const SWFMatrix& mat,
+    const cxform& cx)
   {
   
     const PathVec& path_vec = def->get_paths();
@@ -1613,6 +1639,9 @@ public:
 
     std::vector<PathVec::const_iterator> subshapes = find_subshapes(path_vec);
     
+    const std::vector<fill_style>& fill_styles = def->get_fill_styles();
+    const std::vector<line_style>& line_styles = def->get_line_styles();
+    
     for (size_t i = 0; i < subshapes.size()-1; ++i) {
       PathVec subshape_paths;
       
@@ -1627,7 +1656,7 @@ public:
     }
   }
 
-  virtual void draw_glyph(shape_character_def *def, const matrix& mat,
+  virtual void draw_glyph(shape_character_def *def, const SWFMatrix& mat,
     const rgba& c)
   {
     if (_drawing_mask) abort();
@@ -1693,6 +1722,7 @@ public:
     
 
 private:
+  
   Tesselator _tesselator;
   float _xscale;
   float _yscale;
