@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __GTK_GLUE_H__
-#define __GTK_GLUE_H__
+#ifndef GNASH_GTK_GLUE_H
+#define GNASH_GTK_GLUE_H
 
 #include "gnash.h"
 
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 namespace gnash
 {
@@ -29,23 +29,30 @@ namespace gnash
 class GtkGlue
 {
   public:
-    virtual ~GtkGlue() { };
+    GtkGlue() : _drawing_area(0) { }
+    virtual ~GtkGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
 
     virtual void prepDrawingArea(GtkWidget *drawing_area) = 0;
     virtual render_handler* createRenderHandler() = 0;
-    virtual void setRenderHandlerSize(int /*width*/, int /*height*/) { };
+    virtual void setRenderHandlerSize(int /*width*/, int /*height*/) {}
     virtual void render() = 0;
+    
     virtual void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/)
-			{ render();	};
+    {
+        render();	
+    }
+
     virtual void configure(GtkWidget *const widget,
-                           GdkEventConfigure *const event) = 0;
-    virtual void beforeRendering() { /* nop */ };
+            GdkEventConfigure *const event) = 0;
+    
+    virtual void beforeRendering() {};
+
   protected:
     GtkWidget *_drawing_area;
 };
 
 } // namespace gnash
 
-// end of __GTK_GLUE_H__
+// end of GNASH_GTK_GLUE_H
 #endif
