@@ -1,6 +1,6 @@
-// gui.cpp:  Top level GUI for flash player, for Gnash.
+// gui.cpp:  Top level GUI for SWF player, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,21 +21,22 @@
 #include "gnashconfig.h"
 #endif
 
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "gui.h"
 #include "render.h"  // debug
 #include "render_handler.h"
 #include "sound_handler.h"
-#include "gnash.h" // for get_sound_handler
 #include "movie_root.h"
 #include "VM.h"
 
 #ifdef GNASH_FPS_DEBUG
-#include "tu_timer.h"
+#include "ClockTime.h"
+#include <boost/format.hpp>
 #endif
 
 #include <cstdio>
 #include <cstring>
+#include <algorithm> // std::max, std::min
 
 #ifdef SKIP_RENDERING_IF_LATE
 #include "WallClockTimer.h"
@@ -45,28 +46,19 @@
 /// even if no motion has been detected in the movie.
 //#define FORCE_REDRAW 1
 
-/// Define this to have updated regions enclosed in a red rectangle.
-/// In the future, enabling this might actually use a runtime flag
-/// as an additional conditional.
-/// This has the side effect that all frames will be re-rendered completely
-/// but in contrast to FORCE_REDRAW it won't re-render when no motion
-/// has been detected in the movie (for example when the movie is stopped).
-///
-//#define ENABLE_REGION_UPDATES_DEBUGGING 1
-
 /// Define this if you want to debug the *detection* of region updates only.
 /// This will disable region updates for the backend (GUI+renderer) completely 
 /// so that only the last region (red frame) will be visible. However, this 
 /// slows down rendering as each frame is fully re-rendered. If you want to 
 /// debug the GUI part, however (see if blitting the region works), then you 
 /// probably won't define this.
-#ifdef ENABLE_REGION_UPDATES_DEBUGGING 
+#ifndef DISABLE_REGION_UPDATES_DEBUGGING 
 //#define REGION_UPDATES_DEBUGGING_FULL_REDRAW 1
 #endif 
 
-#ifdef ENABLE_REGION_UPDATES_DEBUGGING
+#ifndef DISABLE_REGION_UPDATES_DEBUGGING
 // a runtime check would make the { x; } block conditionally executed
-#define IF_DEBUG_REGION_UPDATES(x) { x; }
+#define IF_DEBUG_REGION_UPDATES(x) { if (_showUpdatedRegions) { x } }
 #else
 #define IF_DEBUG_REGION_UPDATES(x) 
 #endif
@@ -85,13 +77,18 @@ Gui::Gui() :
     _xid(0),
     _width(1),
     _height(1),
-    _xscale(1.0f),
-    _yscale(1.0f),
     _depth(16),
     _interval(0),
     _renderer(NULL),
     _redraw_flag(true),
-    _fullscreen(false)
+    _fullscreen(false),
+    _mouseShown(true),
+    _maxAdvances(0),
+    _advances(0),
+    _xscale(1.0f),
+    _yscale(1.0f),
+    _xoffset(0),
+    _yoffset(0)
 #ifdef GNASH_FPS_DEBUG
     ,fps_counter(0)
     ,fps_counter_total(0)
@@ -105,6 +102,7 @@ Gui::Gui() :
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
+    ,_showUpdatedRegions(false)
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -112,7 +110,7 @@ Gui::Gui() :
     ,_keyboardMouseMovementsStep(1)
 #endif
 {
-//    GNASH_REPORT_FUNCTION;
+
 }
 
 Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
@@ -121,13 +119,18 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
     _xid(xid),
     _width(1),
     _height(1),
-    _xscale(scale),
-    _yscale(scale),
     _depth(depth),
     _interval(0),
     _renderer(NULL),
     _redraw_flag(true),
-    _fullscreen(false)
+    _fullscreen(false),
+    _mouseShown(true),
+    _maxAdvances(0),
+    _advances(0),
+    _xscale(scale),
+    _yscale(scale),
+    _xoffset(0), // TODO: x and y offset will need update !
+    _yoffset(0)
 #ifdef GNASH_FPS_DEBUG
     ,fps_counter(0)    
     ,fps_counter_total(0)    
@@ -141,6 +144,7 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
+    ,_showUpdatedRegions(false)
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -152,23 +156,58 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
 
 Gui::~Gui()
 {
-//    GNASH_REPORT_FUNCTION;
+    if ( _movieDef.get() ) log_debug("~Gui - _movieDef refcount: %d", _movieDef->get_ref_count());
+
     delete _renderer;
 #ifdef GNASH_FPS_DEBUG
-    printf("Total frame advances: %u\n", fps_counter_total);
+    if ( fps_timer_interval ) {
+        std::cerr << "Total frame advances: " << fps_counter_total << std::endl;
+    }
 #endif
 }
 
 void
 Gui::setFullscreen()
 {
-    log_unimpl("Fullscreen not yet supported in this GUI");
+    log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
 void
 Gui::unsetFullscreen()
 {
-    log_unimpl("Fullscreen not yet supported in this GUI");
+    log_unimpl(_("Fullscreen not yet supported in this GUI"));
+}
+
+void
+Gui::hideMenu()
+{
+    log_unimpl(_("Menu hiding not yet supported in this GUI"));
+}
+
+bool
+Gui::showMouse(bool /* show */)
+{
+	LOG_ONCE(log_unimpl(_("Mouse show/hide not yet supported in this GUI")));
+   	return true;
+}
+
+void
+Gui::showMenu(bool /* show */)
+{
+	LOG_ONCE(log_unimpl(_("menushow not yet supported in this GUI")));
+}
+
+void
+Gui::allowScale(bool allow)
+{
+	if ( ! _stage )
+	{
+		log_error("Gui::allowScale called before a movie_root was available");
+		return;
+	}
+
+	if ( allow ) _stage->setStageScaleMode(movie_root::showAll);
+	else _stage->setStageScaleMode(movie_root::noScale);
 }
 
 void
@@ -183,22 +222,180 @@ Gui::toggleFullscreen()
 }
 
 void
-Gui::menu_restart()
+Gui::restart()
 {
-//    GNASH_REPORT_FUNCTION;
-
 	_stage->reset();
 	_started = false;
 	start();
-
-	//_stage->restart();
-
 }
 
+void
+Gui::updateStageMatrix()
+{
+	if ( ! VM::isInitialized() )
+	{
+		// When VM initializes, we'll get a call to resize_view, which
+		// would call us again.
+		//log_debug("Can't update stage matrix till VM is initialized");
+		return;
+	}
+
+	assert(_stage); // when VM is initialized this should hold
+
+	float swfwidth = _movieDef->get_width_pixels();
+	float swfheight = _movieDef->get_height_pixels();
+
+	// Fetch scale mode
+	movie_root::ScaleMode scaleMode = _stage->getStageScaleMode();
+	switch (scaleMode)
+	{
+		case movie_root::noScale:
+			_xscale = _yscale = 1.0f;
+			break;
+
+		case movie_root::showAll:
+		{
+		
+			// set new scale value ( user-pixel / pseudo-pixel ). Do
+			// not divide by zero, or we end up with an invalid
+			// stage matrix that returns nan values.			
+			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+			
+			// Scale proportionally, using smallest scale
+			if (_xscale < _yscale) _yscale = _xscale;
+			else if (_yscale < _xscale) _xscale = _yscale;
+
+			break;
+		}
+
+		case movie_root::noBorder:
+		{
+
+			// set new scale value ( user-pixel / pseudo-pixel )
+			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+			
+			// Scale proportionally, using biggest scale
+			if (_xscale > _yscale) _yscale = _xscale;
+			else if (_yscale > _xscale) _xscale = _yscale;
+
+			break;
+		}
+
+		case movie_root::exactFit:
+		{
+			// NOTE: changing aspect ratio is valid!
+			_xscale = (swfwidth == 0.0f) ? 1.0f : _width / swfwidth;
+			_yscale = (swfheight == 0.0f) ? 1.0f : _height / swfheight;
+			//LOG_ONCE( log_unimpl("Stage.scaleMode=exactFit") );
+			break;
+		}
+
+		default:
+		{
+			log_error("Invalid scaleMode %d", scaleMode);
+			break;
+		}
+	}
+
+	_xoffset=0;
+	_yoffset=0;
+
+	// Fetch align mode
+	movie_root::StageAlign align = _stage->getStageAlignment();
+	movie_root::StageHorizontalAlign halign = align.first;
+	movie_root::StageVerticalAlign valign = align.second;
+
+	// Handle horizontal alignment
+	switch ( halign )
+	{
+		case movie_root::STAGE_H_ALIGN_L:
+		{
+			// _xoffset=0 is fine
+			break;
+		}
+
+		case movie_root::STAGE_H_ALIGN_R:
+		{
+			// Offsets in pixels
+			float defWidth = swfwidth *= _xscale;
+			float diffWidth = _width-defWidth;
+			_xoffset = diffWidth;
+			break;
+		}
+
+		case movie_root::STAGE_V_ALIGN_C:
+		{
+			// Offsets in pixels
+			float defWidth = swfwidth *= _xscale;
+			float diffWidth = _width-defWidth;
+			_xoffset = diffWidth/2.0;
+			break;
+		}
+
+		default:
+		{
+			log_error("Invalid horizontal align %d", valign);
+			break;
+		}
+	}
+
+	// Handle vertical alignment
+	switch ( valign )
+	{
+		case movie_root::STAGE_V_ALIGN_T:
+		{
+			// _yoffset=0 is fine
+			break;
+		}
+
+		case movie_root::STAGE_V_ALIGN_B:
+		{
+			float defHeight = swfheight *= _yscale;
+			float diffHeight = _height-defHeight;
+			_yoffset = diffHeight;
+			break;
+		}
+
+		case movie_root::STAGE_V_ALIGN_C:
+		{
+			float defHeight = swfheight *= _yscale;
+			float diffHeight = _height-defHeight;
+			_yoffset = diffHeight/2.0;
+			break;
+		}
+
+		default:
+		{
+			log_error("Invalid vertical align %d", valign);
+			break;
+		}
+	}
+
+	//log_debug("updateStageMatrix: scaleMode:%d, valign:%d, halign:%d",
+	//scaleMode, valign, halign);
+
+	// TODO: have a generic set_matrix ?
+	if ( _renderer ) {
+		_renderer->set_scale(_xscale, _yscale);
+		_renderer->set_translation(_xoffset, _yoffset);
+	}
+	else
+	{
+		//log_debug("updateStageMatrix: could not signal updated stage
+		//matrix to renderer (no renderer registered)");
+	}
+
+	// trigger redraw
+	//_redraw_flag |= (_width!=width) || (_height!=height);
+	_redraw_flag = true; // this fixes bug #21971
+}
+
+
 void
 Gui::resize_view(int width, int height)
 {
-//	GNASH_REPORT_FUNCTION;
 
 	assert(width>0);
 	assert(height>0);
@@ -206,147 +403,57 @@ Gui::resize_view(int width, int height)
 	if ( VM::isInitialized() )
 	{
 
-		float swfwidth = _movieDef->get_width_pixels();
-		float swfheight = _movieDef->get_height_pixels();
-
 		if ( _stage && _started )
 		{
 			_stage->set_display_viewport(0, 0, width, height);
 		}
 
-		if ( _stage && _stage->isRescalingAllowed() )
-		{
-			// set new scale value
-			_xscale = width / swfwidth;
-			_yscale = height / swfheight;
-			
-			// always scale proportionally
-			if (_xscale < _yscale) _yscale = _xscale;
-			if (_yscale < _xscale) _xscale = _yscale;
-			
-			_renderer->set_scale(_xscale, _yscale);
-		}
-
 	}
-	else
-	{
-		//log_debug(_("Resize request received while there's still"
-		//	" no movie loaded, can't correctly set movie scale"));
-	}
-	
-	// trigger redraw
-	//_redraw_flag |= (_width!=width) || (_height!=height);
-	_redraw_flag = true; // this fixes bug #21971
 
-	// set new size ?
 	_width = width;
 	_height = height;
-	_validbounds.setTo(0, 0, _width-1, _height-1);
-	//log_debug(_("new size (in twips) is: %dx%d"), _width*20, _height*20); 
+	_validbounds.setTo(0, 0, _width, _height);
+
+	updateStageMatrix();
+
+	if ( _stage && _started ) display(_stage);
 }
 
+
 void
-Gui::menu_quit()
+Gui::toggleSound()
 {
-//    GNASH_REPORT_FUNCTION;
-    quit();
-}
+    assert (_stage);
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runInfo().soundHandler();
 
-void
-Gui::menu_play()
-{
-//    GNASH_REPORT_FUNCTION;
-    //get_current_root()->set_play_state(gnash::sprite_instance::PLAY);
-    play();
-}
+    if (!s) return;
 
-void
-Gui::menu_pause()
-{
-//    GNASH_REPORT_FUNCTION;
-
-    pause();
-//    movie_root* m = get_current_root();
-//    if (m->get_play_state() == gnash::sprite_instance::STOP) {
-//      m->set_play_state(gnash::sprite_instance::PLAY);
-//    } else {
-//      m->set_play_state(gnash::sprite_instance::STOP);
-//    }
-
-}
-
-void
-Gui::menu_stop()
-{
-//    GNASH_REPORT_FUNCTION;
-    //get_current_root()->set_play_state(gnash::sprite_instance::STOP);
-    stop();
-}
-
-void
-Gui::menu_step_forward()
-{
-//    GNASH_REPORT_FUNCTION;
-	movie_root* m = get_current_root();
-	m->goto_frame(m->get_current_frame()+1);
-}
-
-void
-Gui::menu_step_backward()
-{
-//    GNASH_REPORT_FUNCTION;
-
-	movie_root* m = get_current_root();
-	m->goto_frame(m->get_current_frame()-1);
-}
-
-void
-Gui::menu_jump_forward()
-{
-//    GNASH_REPORT_FUNCTION;
-
-	movie_root* m = get_current_root();
-	m->goto_frame(m->get_current_frame()+10);
-}
-
-void
-Gui::menu_jump_backward()
-{
-//    GNASH_REPORT_FUNCTION;
-
-	movie_root* m = get_current_root();
-	m->goto_frame(m->get_current_frame()-10);
-}
-
-void
-Gui::menu_toggle_sound()
-{
-//    GNASH_REPORT_FUNCTION;
-    media::sound_handler* snd_handler = get_sound_handler();
-
-    if (!snd_handler)
-       return;
-
-    if (snd_handler->is_muted()) {
-       snd_handler->unmute();
-    } else {
-       snd_handler->mute();
-    }
+    if (s->is_muted()) s->unmute();
+    else s->mute();
 }
 
 
 void
-Gui::notify_mouse_moved(int x, int y) 
+Gui::notify_mouse_moved(int ux, int uy) 
 {
 	movie_root* m = _stage;
 
-    if ( ! _started ) return;
+	if ( ! _started ) return;
 
-    if ( _stopped ) return;
+	if ( _stopped ) return;
+
+	// A stage pseudopixel is user pixel / _xscale wide
+	boost::int32_t x = (ux-_xoffset) / _xscale;
+
+	// A stage pseudopixel is user pixel / _xscale high
+	boost::int32_t y = (uy-_yoffset) / _yscale;
 
 #ifdef DEBUG_MOUSE_COORDINATES
 	log_debug(_("mouse @ %d,%d"), x, y);
 #endif
+
 	if ( m->notify_mouse_moved(x, y) )
 	{
 		// any action triggered by the
@@ -361,10 +468,14 @@ Gui::notify_mouse_moved(int x, int y)
 		{
 			setCursor(CURSOR_INPUT);
 		}
-		else
+		else if ( activeEntity->allowHandCursor() )
 		{
 			setCursor(CURSOR_HAND);
 		}
+		else
+		{
+			setCursor(CURSOR_NORMAL);
+		}
 	}
 	else
 	{
@@ -372,8 +483,8 @@ Gui::notify_mouse_moved(int x, int y)
 	}
 
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS
-	_xpointer = x;
-	_ypointer = y;
+	_xpointer = ux;
+	_ypointer = uy;
 #endif
 
 
@@ -426,17 +537,17 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 			}
 		}
 		
-		if (modifier & gnash::key::MOD_CONTROL)
+		if (modifier & gnash::key::GNASH_MOD_CONTROL)
 		{
 			switch(k)
 			{
 				case gnash::key::r:
 				case gnash::key::R:
-					menu_restart();
+					restart();
 					break;
 				case gnash::key::p:
 				case gnash::key::P:
-					menu_pause();
+					pause();
 					break;
 				case gnash::key::l:
 				case gnash::key::L:
@@ -446,18 +557,37 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 				case gnash::key::Q:
 				case gnash::key::w:
 				case gnash::key::W:
-					menu_quit();
+					quit();
 					break;
 				case gnash::key::f:
 				case gnash::key::F:
 					toggleFullscreen();
 					break;
-				case gnash::key::RIGHT_BRACKET:
-					menu_step_forward();
+				case gnash::key::MINUS:
+				{
+					// Max interval allowed: 1 second (1FPS)
+					unsigned int ni = std::min(_interval+2, 1000u);
+					setInterval(ni);
 					break;
-				case gnash::key::LEFT_BRACKET:
-					menu_step_backward();
+                }
+				case gnash::key::PLUS:
+				{
+					// Min interval allowed: 1/100 second (100FPS)
+					unsigned int ni = std::max(_interval-2, 10u);
+					setInterval(ni);
 					break;
+                                }
+				case gnash::key::EQUALS:
+				{
+					if ( _stage )
+					{
+						float fps = _stage->get_movie_definition()->get_frame_rate();
+						// Min interval allowed: 1/100 second (100FPS)
+						unsigned int ni = 1000.0/fps;
+						setInterval(ni);
+					}
+					break;
+                                }
 				default:
 					break;
 			}
@@ -466,7 +596,7 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 			if ( _keyboardMouseMovements )
 			{
 				int step = _keyboardMouseMovementsStep; 
-				if (modifier & gnash::key::MOD_SHIFT) step*=5; // x5 if SHIFT is pressed
+				if (modifier & gnash::key::GNASH_MOD_SHIFT) step*=5; // x5 if SHIFT is pressed
 				switch(k)
 				{
 					case gnash::key::UP:
@@ -579,9 +709,37 @@ Gui::display(movie_root* m)
 	{
 		changed_ranges.setWorld();
 	}
+	
+	// DEBUG ONLY:
+  // This is a good place to inspect the invalidated bounds state. Enable
+  // the following block (and parts of it) if you need to. 
+#if 0
+  {
+    // This may print a huge amount of information, but is useful to analyze
+    // the (visible) object structure of the movie and the flags of the
+    // characters. For example, a characters should have set the 
+    // m_child_invalidated flag if at least one of it's childs has the
+    // invalidated flag set.
+    log_debug("DUMPING CHARACTER TREE"); 
+    
+    InfoTree tr;
+    InfoTree::iterator top = tr.begin();
+    _stage->getMovieInfo(tr, top);
+
+    for (InfoTree::iterator i = tr.begin(), e = tr.end();
+            i != e; ++i) {
+        std::cout << std::string(tr.depth(i) * 2, ' ') << i->first << ": " << 
+            i->second << std::endl;
+    }
+
+
+    // less verbose, and often necessary: see the exact coordinates of the
+    // invalidated bounds (mainly to see if it's NULL or something else).	
+    std::cout << "Calculated changed ranges: " << changed_ranges << "\n";
+  }
+#endif
   
 	// Avoid drawing of stopped movies
-
 	if ( ! changed_ranges.isNull() ) // use 'else'?
 	{
 		// Tell the GUI(!) that we only need to update this
@@ -616,7 +774,7 @@ Gui::display(movie_root* m)
 		if ( ! changed_ranges.isWorld() )
 		{
 		
-			for (int rno=0; rno<changed_ranges.size(); rno++) {
+			for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
 			
 				geometry::Range2d<float> bounds = changed_ranges.getRange(rno);
 
@@ -634,8 +792,9 @@ Gui::display(movie_root* m)
 				corners[2].y = ymax;
 				corners[3].x = xmin;
 				corners[3].y = ymax;
-				gnash::render::draw_poly(&corners[0], 4,
-					rgba(0,0,0,0), rgba(255,0,0,255), matrix::identity, false);
+				SWFMatrix no_transform;
+				gnash::render::draw_poly(corners, 4,
+					rgba(0,0,0,0), rgba(255,0,0,255), no_transform, false);
 					
 			}
 		}
@@ -656,14 +815,35 @@ Gui::play()
 
     _stopped = false;
     if ( ! _started ) start();
+    else
+    {
+        assert (_stage);
+        // @todo since we registered the sound handler, shouldn't we know
+        //       already what it is ?!
+        sound::sound_handler* s = _stage->runInfo().soundHandler();
+        if ( s ) s->unpause();
+    }
+
+    playHook ();
 }
 
 void
 Gui::stop()
 {
+    // _stage must be registered before this is called.
+    assert(_stage);
+
     if ( _stopped ) return;
+    if ( isFullscreen() ) unsetFullscreen();
 
     _stopped = true;
+
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runInfo().soundHandler();
+    if ( s ) s->pause();
+
+    stopHook();
 }
 
 void
@@ -673,7 +853,16 @@ Gui::pause()
     {
         play();
     }
-    else _stopped = true;
+    else
+    {
+        // @todo since we registered the sound handler, shouldn't we know
+        //       already what it is ?!
+    	sound::sound_handler* s = _stage->runInfo().soundHandler();
+    	if ( s ) s->pause();
+        _stopped = true;
+
+        stopHook();
+    }
 }
 
 void
@@ -690,23 +879,27 @@ Gui::start()
     mr->setVariables(_flashVars);
 
     _stage->setRootMovie( mr.release() ); // will construct the instance
-    _stage->set_display_viewport(0, 0, _width, _height);
+    resize_view(_width, _height); // to properly update stageMatrix if scaling is given 
 
     bool background = true; // ??
     _stage->set_background_alpha(background ? 1.0f : 0.05f);
 
-
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runInfo().soundHandler();
+    if ( s ) s->unpause();
     _started = true;
 }
 
 bool
 Gui::advanceMovie()
 {
+
 	if ( isStopped() ) return true;
 
     if ( ! _started ) start();
   
-//	GNASH_REPORT_FUNCTION;
+
 
 #ifdef SKIP_RENDERING_IF_LATE
 	WallClockTimer advanceTimer;
@@ -731,7 +924,7 @@ Gui::advanceMovie()
 	m->advance();
 	m->get_movie_definition()->ensure_frame_loaded(tot_frames);
 	m->goto_frame(cur_frame+1);
-    	m->set_play_state(gnash::sprite_instance::PLAY);
+    	m->set_play_state(gnash::MovieClip::PLAY);
 	log_debug(_("Frame %d"), m->get_current_frame());
 #endif
 
@@ -780,13 +973,19 @@ Gui::advanceMovie()
 	if ( ! loops() )
 	{
 		size_t curframe = m->get_current_frame(); // can be 0 on malformed SWF
-		gnash::sprite_instance* si = m->getRootMovie();
+		gnash::MovieClip* si = m->getRootMovie();
 		if (curframe + 1 >= si->get_frame_count())
 		{
 			quit(); 
 		}
 	}
 
+    /// Quit if we've reached the advance limit.
+    if (_maxAdvances && (_advances++ > _maxAdvances))
+    {
+        quit();
+    }
+
 	return true;
 }
 
@@ -801,24 +1000,6 @@ Gui::want_redraw()
     return false;
 }
 
-float
-Gui::getXScale()
-{
-    return _xscale;
-}
-
-float
-Gui::getYScale()
-{
-    return _yscale;
-}
-
-bool
-Gui::loops()
-{
-    return _loop;
-}
-
 void
 Gui::setInvalidatedRegion(const rect& /*bounds*/)
 {
@@ -841,75 +1022,118 @@ Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 	setInvalidatedRegion(bounds);
 }
 
+#ifdef USE_SWFTREE
+
 std::auto_ptr<Gui::InfoTree>
 Gui::getMovieInfo() const
 {
-    std::auto_ptr<InfoTree> ret;
+    std::auto_ptr<InfoTree> tr;
 
     if ( ! VM::isInitialized() )
     {
-        return ret;
+        return tr;
     }
 
-    ret.reset(new InfoTree());
+    tr.reset(new InfoTree());
 
-    VM& vm = VM::get();
+    // Top nodes for the tree:
+    // 1. VM information
+    // 2. "Stage" information
+    // 3. ...
 
-    // Print VM version
-    int vmSWFVersion = vm.getSWFVersion();
-    char buf[16];
-    snprintf(buf, 16, "SWF%d", vmSWFVersion); buf[15] = '\0';
-    ret->insert(ret->begin(), StringPair("VM", buf));
+    InfoTree::iterator topIter = tr->begin();
+    InfoTree::iterator firstLevelIter;
+
+    VM& vm = _stage->getVM();
+
+    std::ostringstream os;
+
+    //
+    /// VM top level
+    //
+    os << "SWF " << vm.getSWFVersion();
+    topIter = tr->insert(topIter, StringPair("VM version", os.str()));
+
+    // This short-cut is to avoid a bug in movie_root's getMovieInfo,
+    // which relies on the availability of a _rootMovie for doing
+    // it's work, while we don't set it if we didn't start..
+    // 
+    if ( ! _started )
+    {
+        topIter = tr->insert(topIter, StringPair("Stage properties", 
+                    "not constructed yet"));
+        return tr;
+    }
 
-    // Print info about levels (only originating movie for now, then will be extended)
     movie_root& stage = vm.getRoot();
-    boost::intrusive_ptr<movie_instance> level0 = stage.getRootMovie();
-    movie_definition* def0 = level0->get_movie_definition();
-    assert(def0);
-    snprintf(buf, 16, "SWF%d", def0->get_version()); buf[15] = '\0';
-    ret->insert(ret->begin(), StringPair("_level0 SWFVersion", std::string(buf)));
-    ret->insert(ret->begin(), StringPair("_level0 URL", def0->get_url()));
+    stage.getMovieInfo(*tr, topIter);
 
-    // Print info about scripts state (enabled/disabled)
-    ret->insert(ret->begin(), StringPair("Stage scripts", stage.scriptsDisabled() ? " disabled" : "enabled"));
+    //
+    /// Mouse entities
+    //
+    topIter = tr->insert(topIter, StringPair("Mouse Entities", ""));
 
-    // Print info about mouse entities
-
-    using std::string;
     const character* ch;
-
     ch = stage.getActiveEntityUnderPointer();
     if ( ch )
     {
-	std::stringstream ss;
-	ss << ch->getTarget() << " (" + typeName(*ch) << " - id:" << ch->get_id() << " depth:" << ch->get_depth();
-    	ret->insert(ret->begin(), StringPair("Active mouse entity: ", ss.str()));
+	    std::stringstream ss;
+	    ss << ch->getTarget() << " (" + typeName(*ch)
+            << " - id:" << ch->get_id()
+            << " - depth:" << ch->get_depth()
+            << " - useHandCursor:" << ch->allowHandCursor()
+            << ")";
+    	firstLevelIter = tr->append_child(topIter, StringPair("Active entity under mouse pointer", ss.str()));
     }
 
     ch = stage.getEntityUnderPointer();
     if ( ch )
     {
-	std::stringstream ss;
-	ss << ch->getTarget() << " (" + typeName(*ch) << " - id:" << ch->get_id() << " depth:" << ch->get_depth();
-    	ret->insert(ret->begin(), StringPair("Topmost mouse entity: ", ss.str()));
+	    std::stringstream ss;
+	    ss << ch->getTarget() << " (" + typeName(*ch) 
+               << " - id:" << ch->get_id() << " - depth:" << ch->get_depth()
+               << ")";
+	firstLevelIter = tr->append_child(topIter, StringPair("Topmost entity under mouse pointer", ss.str()));
     }
 
     ch = stage.getDraggingCharacter();
     if ( ch ) 
     {
-	std::stringstream ss;
-	ss << ch->getTarget() << " (" + typeName(*ch) << " - id:" << ch->get_id() << " depth:" << ch->get_depth();
-    	ret->insert(ret->begin(), StringPair("Dragging character: ", ss.str()));
+	    std::stringstream ss;
+	    ss << ch->getTarget() << " (" + typeName(*ch) 
+               << " - id:" << ch->get_id()
+               << " - depth:" << ch->get_depth() << ")";
+    	firstLevelIter = tr->append_child(topIter, StringPair("Dragging character: ", ss.str()));
     }
 
-    return ret;
+    //
+    /// GC row
+    //
+    topIter = tr->insert(topIter, StringPair("GC Statistics", ""));
+    GC::CollectablesCount cc;
+    GC::get().countCollectables(cc);
+    
+    const std::string lbl = "GC managed ";
+    for (GC::CollectablesCount::iterator i=cc.begin(), e=cc.end(); i!=e; ++i)
+    {
+        const std::string& typ = i->first;
+        std::ostringstream ss;
+        ss << i->second;
+        firstLevelIter = tr->append_child(topIter,
+                            StringPair(lbl + typ, ss.str()));
+    }
+
+    tr->sort(firstLevelIter.begin(), firstLevelIter.end());
+
+    return tr;
 }
 
+#endif
+
 #ifdef GNASH_FPS_DEBUG
 void 
 Gui::fpsCounterTick()
 {
-  // GNASH_REPORT_FUNCTION;
 
   // increment this *before* the early return so that
   // frame count on exit is still valid
@@ -920,7 +1144,7 @@ Gui::fpsCounterTick()
 	  return;
   }
 
-  boost::uint64_t current_timer = tu_timer::get_ticks();
+  boost::uint64_t current_timer = clocktime::getTicks();
 
   // TODO: keep fps_timer_interval in milliseconds to avoid the multiplication
   //       at each fpsCounterTick call...
@@ -944,7 +1168,7 @@ Gui::fpsCounterTick()
       // the timers are unsigned, so when the clock runs "backwards" it leads
       // to a very high difference value. In theory, this should never happen
       // with ticks, but it does on my machine (which may have a hw problem?).
-      printf("Time glich detected, need to restart FPS counters, sorry...\n");
+      std::cerr << "Time glitch detected, need to restart FPS counters, sorry..." << std::endl;
       
       fps_timer = current_timer;
       fps_start_timer = current_timer;
@@ -958,16 +1182,18 @@ Gui::fpsCounterTick()
       fps_rate_min = rate;
       fps_rate_max = rate; 
     } else {
-      fps_rate_min = fmin(fps_rate_min, rate);
-      fps_rate_max = fmax(fps_rate_max, rate);
+      fps_rate_min = std::min<float>(fps_rate_min, rate);
+      fps_rate_max = std::max<float>(fps_rate_max, rate);
     }
     
     float avg = fps_counter_total / secs_total; 
   
     //log_debug("Effective frame rate: %0.2f fps", (float)(fps_counter/secs));
-    printf("Effective frame rate: %0.2f fps (min %0.2f, avg %0.2f, max %0.2f, "
-      "%u frames in %0.1f secs total)\n", rate, fps_rate_min, avg, fps_rate_max,
-      fps_counter_total, secs_total);
+    std::cerr << boost::format("Effective frame rate: %0.2f fps "
+                               "(min %0.2f, avg %0.2f, max %0.2f, "
+                               "%u frames in %0.1f secs total)") % rate %
+                               fps_rate_min % avg % fps_rate_max %
+                               fps_counter_total % secs_total << std::endl;
       
     fps_counter = 0;
     fps_timer = current_timer;
@@ -1001,5 +1227,12 @@ Gui::setStage(movie_root* stage)
     _stage = stage;
 }
 
+bool
+Gui::yesno(const std::string& question)
+{
+    log_error("This gui didn't override 'yesno', assuming 'yes' answer to question: %s", question);
+    return true;
+}
+
 // end of namespace
 }
