@@ -1,6 +1,6 @@
 // GC.h: Garbage Collector for Gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,12 +24,16 @@
 #endif
 
 #include <list>
+#include <map>
+#include <string>
 #include <algorithm> //for std::find
 
 #ifndef NDEBUG
 # include <boost/thread.hpp>
 #endif
 
+#include "dsodefs.h"
+
 // Define the following macro to enable GC verbosity 
 // Verbosity levels:
 //   1 - print stats about how many resources are registered and how many 
@@ -181,7 +185,7 @@ private:
 /// itself.
 ///
 ///
-class GC
+class DSOEXPORT GC
 {
 
 public:
@@ -234,7 +238,7 @@ public:
 
 		_resList.push_back(item);
 #if GNASH_GC_DEBUG > 1
-		log_debug(_("GC %p: collectable %p added, num collectables: " SIZET_FMT), (void*)this, (void*)item, _resList.size());
+		log_debug(_("GC %p: collectable %p added, num collectables: %d"), (void*)this, (void*)item, _resList.size());
 #endif
 	}
 
@@ -245,12 +249,16 @@ public:
 	///
 	void collect();
 
+	typedef std::map<std::string, unsigned int> CollectablesCount;
+
+	/// Count collectables
+	void countCollectables(CollectablesCount& count) const;
+
 private:
 
 	/// Number of newly registered collectable since last collection run
 	/// triggering next collection.
-	/// Should be made a parameter ?
-	static const unsigned int maxNewCollectablesCount = 10;
+	static unsigned int maxNewCollectablesCount;
 
 	/// Create a garbage collector, using the given root
 	GC(GcRoot& root)
@@ -282,7 +290,10 @@ private:
 	}
 
 	/// Delete all unreachable objects, and mark the others unreachable again
-	void cleanUnreachable();
+	//
+	/// @return number of objects deleted
+	///
+	size_t cleanUnreachable();
 
 	ResList _resList;
 
