@@ -1,6 +1,6 @@
 // utf8.h: utilities for converting to and from UTF-8
 // 
-//   Copyright (C) 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "dsodefs.h" // For DSOEXPORT
 #include <string>
 #include <boost/cstdint.hpp> // for C99 int types
+#include <limits>
 
 /// Utilities to convert between std::string and std::wstring.
 //
@@ -53,7 +54,7 @@
 /// gnash::edit_text_character, ord() and chr().
 namespace utf8
 {
-    static const boost::uint32_t invalid = -1;
+    static const boost::uint32_t invalid = std::numeric_limits<boost::uint32_t>::max();
 
 	/// Converts a std::string with multibyte characters into a std::wstring.
 	//
@@ -95,7 +96,7 @@ namespace utf8
 	//
 	/// Allows storage of Latin1 (ISO-8859-1) characters. This
 	/// is the format of SWF5 and below.
-	std::string encodeLatin1Character(boost::uint32_t ucsCharacter);
+	DSOEXPORT std::string encodeLatin1Character(boost::uint32_t ucsCharacter);
 
 	enum TextEncoding {
 		encUNSPECIFIED,
