@@ -1,6 +1,6 @@
 // SWFMovieDefinition.cpp: load a SWF definition
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,43 +21,34 @@
 #include "gnashconfig.h" // USE_SWFTREE
 #endif
 
+#include "GnashSleep.h"
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "SWFMovieDefinition.h"
 #include "movie_definition.h" // for inheritance
-#include "sprite_instance.h" // for ??
+#include "MovieClip.h" // for ??
 #include "zlib_adapter.h"
 #include "IOChannel.h" // for use
 #include "SWFStream.h"
 #include "GnashImageJpeg.h"
-//#include "fontlib.h"
-#include "font.h"
+#include "RunInfo.h"
+#include "Font.h"
 #include "log.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "movie_instance.h"
-#include "bitmap_character_def.h"
 #include "swf/TagLoadersTable.h"
 #include "movie_root.h"
 #include "VM.h" // for assertions
 #include "GnashException.h" // for parser exception
 #include "ControlTag.h"
 #include "sound_definition.h" // for sound_sample
-#include "GnashSleep.h"
+#include "ExportableResource.h"
+
 #include <boost/bind.hpp>
 #include <boost/version.hpp>
-
 #include <iomanip>
 #include <memory>
 #include <string>
 #include <algorithm> // std::make_pair
-#include <unistd.h>
-
-
-// Increment this when the cache data format changes.
-#define CACHE_FILE_VERSION 4
-
-// If != 0 this is the number of frames to load at each iteration
-// of the main loop. Loading in chunks greatly speeds the process up
-#define FRAMELOAD_CHUNK 0
 
 // Debug frames load
 #undef DEBUG_FRAMES_LOAD
@@ -70,7 +61,7 @@
 //#undef DEBUG_THREADS_LOCKING
 
 // Define this to get debugging output for symbol library use
-#define DEBUG_EXPORTS
+//#define DEBUG_EXPORTS
 
 namespace gnash
 {
@@ -85,8 +76,6 @@ MovieLoader::MovieLoader(SWFMovieDefinition& md)
 
 MovieLoader::~MovieLoader()
 {
-	//cout << "MovieLoader dtor called" << endl;
-
 	// we should assert _movie_def._loadingCanceled
 	// but we're not friend yet (anyone introduce us ?)
 	if ( _thread.get() )
@@ -133,20 +122,20 @@ bool
 MovieLoader::start()
 {
 #ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	// don't start MovieLoader thread !
-	abort();
+    std::abort();
 #endif
-	// We have two sanity checks, started() and isSelfThread() which rely
+	// don't start MovieLoader thread() which rely
 	// on boost::thread() returning before they are executed. Therefore,
 	// we must employ locking.
 	// Those tests do seem a bit redundant, though...
 	boost::mutex::scoped_lock lock(_mutex);
 
-	_thread.reset( new boost::thread(boost::bind(execute, boost::ref(*this), &_movie_def)) );
+	_thread.reset(new boost::thread(boost::bind(
+                    execute, boost::ref(*this), &_movie_def)));
 
 	_barrier.wait(); // let execution start befor returning
 
-	return true;
+    return true;
 }
 
 //
@@ -156,19 +145,22 @@ MovieLoader::start()
 /// Log the contents of the current tag, in hex to the output strream
 static void	dumpTagBytes(SWFStream& in, std::ostream& os)
 {
-    const unsigned int rowlength = 16;
+    const std::streamsize rowlength = 16;
     os << std::endl;
     
     // This decremented until we reach the end of the stream.
-    unsigned int toRead = in.get_tag_end_position() - in.tell();
+    std::streamsize toRead = in.get_tag_end_position() - in.tell();
     in.ensureBytes(toRead);
 
     unsigned char buf[rowlength];    
     while (toRead)
     {
         // Read in max row length or remainder of stream.
-        const unsigned int thisRow = std::min<unsigned int>(toRead, rowlength);
-        const unsigned int got = in.read(reinterpret_cast<char*>(&buf), thisRow);
+        const std::streamsize thisRow = 
+            std::min<std::streamsize>(toRead, rowlength);
+        
+        const std::streamsize got = 
+            in.read(reinterpret_cast<char*>(&buf), thisRow);
         
         // Check that we read all the bytes we expected.
         if (got < thisRow)
@@ -191,7 +183,7 @@ static void	dumpTagBytes(SWFStream& in, std::ostream& os)
 // SWFMovieDefinition
 //
 
-SWFMovieDefinition::SWFMovieDefinition()
+SWFMovieDefinition::SWFMovieDefinition(const RunInfo& runInfo)
 	:
 	// FIXME: use a class-static TagLoadersTable for SWFMovieDefinition
 #ifdef USE_SWFTREE
@@ -209,7 +201,8 @@ SWFMovieDefinition::SWFMovieDefinition()
 	m_file_length(0),
 	m_jpeg_in(0),
 	_loader(*this),
-	_loadingCanceled(false)
+	_loadingCanceled(false),
+    _runInfo(runInfo)
 {
 }
 
@@ -220,15 +213,16 @@ SWFMovieDefinition::~SWFMovieDefinition()
 	_loadingCanceled = true;
 
 	// Release frame tags
-	for (PlayListMap::iterator i=m_playlist.begin(), e=m_playlist.end(); i!=e; ++i)
+	for (PlayListMap::iterator i = m_playlist.begin(),
+            e = m_playlist.end(); i != e; ++i)
 	{
 		PlayList& pl = i->second;
 
-		for (PlayList::iterator j=pl.begin(), je=pl.end(); j!=je; ++j)
+		for (PlayList::iterator j = pl.begin(), je = pl.end(); j!=je; ++j)
 		{
-                    delete *j;
-                }
+            delete *j;
         }
+    }
 
 	// It's supposed to be cleaned up in read()
 	// TODO: join with loader thread instead ?
@@ -248,60 +242,58 @@ SWFMovieDefinition::get_character_def(int character_id)
 
 	boost::mutex::scoped_lock lock(_dictionaryMutex);
 
-	boost::intrusive_ptr<character_def> ch = _dictionary.get_character(character_id);
+	boost::intrusive_ptr<character_def> ch = 
+        _dictionary.get_character(character_id);
 #ifndef GNASH_USE_GC
 	assert(ch == NULL || ch->get_ref_count() > 1);
-#endif // ndef GNASH_USE_GC
+#endif 
 	return ch.get(); // mm... why don't we return the boost::intrusive_ptr?
 }
 
-void SWFMovieDefinition::add_font(int font_id, font* f)
+void SWFMovieDefinition::add_font(int font_id, Font* f)
 {
     assert(f);
-    m_fonts.insert(std::make_pair(font_id, boost::intrusive_ptr<font>(f)));
+    m_fonts.insert(std::make_pair(font_id, boost::intrusive_ptr<Font>(f)));
 }
 
-font* SWFMovieDefinition::get_font(int font_id) const
+Font* SWFMovieDefinition::get_font(int font_id) const
 {
 
     FontMap::const_iterator it = m_fonts.find(font_id);
     if ( it == m_fonts.end() ) return NULL;
-    boost::intrusive_ptr<font> f = it->second;
+    boost::intrusive_ptr<Font> f = it->second;
     assert(f->get_ref_count() > 1);
     return f.get();
 }
 
-font*
-SWFMovieDefinition::get_font(const std::string& name, bool bold, bool italic) const
+Font*
+SWFMovieDefinition::get_font(const std::string& name, bool bold, bool italic)
+    const
 {
 
     for (FontMap::const_iterator it=m_fonts.begin(), itEnd=m_fonts.end(); it != itEnd; ++it)
     {
-       font* f = it->second.get();
+       Font* f = it->second.get();
        if ( f->matches(name, bold, italic) ) return f;
     }
     return 0;
 }
 
-bitmap_character_def* SWFMovieDefinition::get_bitmap_character_def(int character_id)
+BitmapInfo*
+SWFMovieDefinition::getBitmap(int character_id)
 {
-    BitmapMap::iterator it = m_bitmap_characters.find(character_id);
-    if ( it == m_bitmap_characters.end() ) return NULL;
-    else return it->second.get();
+    Bitmaps::iterator it = _bitmaps.find(character_id);
+    if (it == _bitmaps.end()) return 0;
+    
+    return it->second.get();
 }
 
 void
-SWFMovieDefinition::add_bitmap_character_def(int character_id,
-		bitmap_character_def* ch)
+SWFMovieDefinition::addBitmap(int id, boost::intrusive_ptr<BitmapInfo> im)
 {
-    assert(ch);
-    //log_debug(_("Add bitmap character %d"), character_id);
-    //m_bitmap_characters.add(character_id, ch);
-    m_bitmap_characters.insert(std::make_pair(character_id, boost::intrusive_ptr<bitmap_character_def>(ch)));
+    assert(im);
+    _bitmaps.insert(std::make_pair(id, im));
 
-	// we can *NOT* generate bitmap_info until
-	// a renderer is present
-    add_bitmap_info(ch->get_bitmap_info());
 }
 
 sound_sample* SWFMovieDefinition::get_sound_sample(int character_id)
@@ -312,7 +304,7 @@ sound_sample* SWFMovieDefinition::get_sound_sample(int character_id)
     boost::intrusive_ptr<sound_sample> ch = it->second;
 #ifndef GNASH_USE_GC
     assert(ch->get_ref_count() > 1);
-#endif // ndef GNASH_USE_GC
+#endif 
 
     return ch.get();
 }
@@ -323,23 +315,23 @@ void SWFMovieDefinition::add_sound_sample(int character_id, sound_sample* sam)
     IF_VERBOSE_PARSE(
     log_parse(_("Add sound sample %d assigning id %d"),
 		character_id, sam->m_sound_handler_id);
-    );
+    )
     m_sound_samples.insert(std::make_pair(character_id,
 			    boost::intrusive_ptr<sound_sample>(sam)));
 }
 
 // Read header and assign url
 bool
-SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in, const std::string& url)
+SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in,
+        const std::string& url)
 {
 
 	_in = in;
 
 	// we only read a movie once
-	assert(_str.get() == NULL);
+	assert(!_str.get());
 
-	if ( url == "" ) _url = "<anonymous>";
-	else _url = url;
+	_url = url.empty() ? "<anonymous>" : url;
 
 	boost::uint32_t file_start_pos = _in->tell();
 	boost::uint32_t header = _in->read_le32();
@@ -355,21 +347,19 @@ SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in, const std::string& u
 			"file does not start with a SWF header"));
 		return false;
         }
-	bool	compressed = (header & 255) == 'C';
+	const bool compressed = (header & 255) == 'C';
 
 	IF_VERBOSE_PARSE(
-		log_parse(_("version = %d, file_length = %d"),
-			m_version, m_file_length);
-	);
+		log_parse(_("version: %d, file_length: %d"), m_version, m_file_length);
+    )
 
-	if ( m_version > 7 )
+	if (m_version > 7)
 	{
 		log_unimpl(_("SWF%d is not fully supported, trying anyway "
 			"but don't expect it to work"), m_version);
 	}
 
-	if (compressed)
-        {
+	if (compressed) {
 #ifndef HAVE_ZLIB_H
 		log_error(_("SWFMovieDefinition::read(): unable to read "
 			"zipped SWF data; gnash was compiled without zlib support"));
@@ -382,7 +372,7 @@ SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in, const std::string& u
 		// Uncompress the input as we read it.
 		_in = zlib_adapter::make_inflater(_in);
 #endif
-        }
+    }
 
 	assert(_in.get());
 
@@ -400,7 +390,8 @@ SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in, const std::string& u
 	}
 
 	// It seems frame rate is limited to a max 
-	// 84 was found by testing the pp, might be turned into a compile-time define
+	// 84 was found by testing the pp, might be turned into a
+    // compile-time define
 	static const int maxfps = 84;
 	
 	_str->ensureBytes(2 + 2); // frame rate, frame count.
@@ -423,10 +414,8 @@ SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in, const std::string& u
 
 	m_frame_count = _str->read_u16();
 
-	/* Markus: Probably this is better anyways */
-
 	// TODO: This seems dangerous, check closely
-	if(m_frame_count == 0) m_frame_count++;
+	if (!m_frame_count) ++m_frame_count;
 
 	IF_VERBOSE_PARSE(
 		log_parse(_("frame size = %s, frame rate = %f, frames = %d"),
@@ -450,7 +439,7 @@ SWFMovieDefinition::completeLoad()
 	assert ( VM::isInitialized() );
 
 	// should call readHeader before this
-	assert( _str.get() != NULL );
+	assert(_str.get());
 
 #ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
 
@@ -477,16 +466,6 @@ SWFMovieDefinition::completeLoad()
 	return true;
 }
 
-// Read a .SWF movie.
-bool
-SWFMovieDefinition::read(std::auto_ptr<IOChannel> in, const std::string& url)
-{
-
-	if ( ! readHeader(in, url) ) return false;
-
-	return completeLoad();
-}
-
 
 // 1-based frame number
 bool
@@ -495,19 +474,16 @@ SWFMovieDefinition::ensure_frame_loaded(size_t framenum)
 	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
 
 #ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
-	return ( framenum <= _frames_loaded );
+	return (framenum <= _frames_loaded);
 #endif
 
 	if ( framenum <= _frames_loaded ) return true;
 
 	_waiting_for_frame = framenum;
-        //log_debug(_("Waiting for frame %u to be loaded"), framenum);
 
 	// TODO: return false on timeout
 	_frame_reached_condition.wait(lock);
 
-        //log_debug(_("Condition reached (_frames_loaded=%u)"), _frames_loaded);
-
 	return ( framenum <= _frames_loaded );
 }
 
@@ -526,8 +502,8 @@ std::ostream&
 operator<<(std::ostream& o, const CharacterDictionary& cd)
 {
 
-   	for (CharacterDictionary::CharacterConstIterator it = cd.begin(), endIt = cd.end();
-   	        it != endIt; it++)
+   	for (CharacterDictionary::CharacterConstIterator it = cd.begin(), 
+            endIt = cd.end(); it != endIt; it++)
    	{
    	    o << std::endl
    	      << "Character: " << it->first
@@ -544,65 +520,27 @@ CharacterDictionary::get_character(int id)
 	if ( it == _map.end() )
 	{
 		IF_VERBOSE_PARSE(
-		log_parse(_("Could not find char %d, dump is: %s"), id, *this);
+            log_parse(_("Could not find char %d, dump is: %s"), id, *this);
 		);
 		return boost::intrusive_ptr<character_def>();
 	}
-	else return it->second;
+	
+    return it->second;
 }
 
 void
-CharacterDictionary::add_character(int id, boost::intrusive_ptr<character_def> c)
+CharacterDictionary::add_character(int id,
+        boost::intrusive_ptr<character_def> c)
 {
 	//log_debug(_("CharacterDictionary: add char %d"), id);
 	_map[id] = c;
 }
 
-// Load next chunk of this sprite frames.
-// This is possibly better defined in movie_definition
-void
-SWFMovieDefinition::load_next_frame_chunk()
-{
-
-	size_t framecount = get_frame_count();
-	size_t lastloaded = get_loading_frame();
-
-	// nothing to do
-	if ( lastloaded == framecount ) return;
-
-	size_t nextframe = lastloaded+1;
-
-#if FRAMELOAD_CHUNK
-	nextframe += FRAMELOAD_CHUNK; // load in chunks of 10 frames
-	if ( nextframe > framecount ) nextframe = framecount;
-#endif
-	//log_debug(_("Framecount: %u, Lastloaded: %u"), framecount, lastloaded);
-	if ( nextframe <= framecount )
-	{
-#ifdef DEBUG_FRAMES_LOAD // debugging
-		log_debug(_("Ensure load of frame %u/%u (last loaded is: %u)"),
-			nextframe, framecount, lastloaded);
-#endif
-		if ( ! ensure_frame_loaded(nextframe) )
-		{
-			log_error(_("Could not advance to frame %d"),
-				nextframe);
-			// these kind of errors should be handled by callers
-			abort();
-		}
-	}
-#ifdef DEBUG_FRAMES_LOAD
-	else
-	{
-		log_debug(_("No more frames to load. Framecount: %u, Lastloaded: %u, next to load: %u"), framecount, lastloaded, nextframe);
-	}
-#endif
-}
 
 void
 SWFMovieDefinition::read_all_swf()
 {
-	assert(_str.get() != NULL);
+	assert(_str.get());
 
 #ifdef LOAD_MOVIES_IN_A_SEPARATE_THREAD
 	assert( _loader.isSelfThread() );
@@ -616,87 +554,81 @@ SWFMovieDefinition::read_all_swf()
 
 	try {
 
-	//size_t it=0;
-	while ( (boost::uint32_t) str.tell() < _swf_end_pos )
-	{
-		if ( _loadingCanceled )
-		{
-			log_debug("Loading thread cancelation requested, returning from read_all_swf");
-			return;
-		}
+        while (str.tell() < _swf_end_pos)
+        {
+            if (_loadingCanceled)
+            {
+                log_debug("Loading thread cancelation requested, "
+                        "returning from read_all_swf");
+                return;
+            }
 
-		//log_debug(_("Loading thread iteration %u"), it++);
-
-		SWF::tag_type tag_type = str.open_tag();
+            SWF::TagType tag = str.open_tag();
 
 parse_tag:
 
-		if (tag_type == SWF::END)
-                {
-			if ((unsigned int) str.tell() != _swf_end_pos)
-                        {
+            if (tag == SWF::END) {
+                if (str.tell() != _swf_end_pos) {
 		    		IF_VERBOSE_MALFORMED_SWF(
-				// Safety break, so we don't read past
-				// the end of the  movie.
-				log_swferror(_("Hit stream-end tag, "
-					"but not at the advertised SWF end; "
-					"stopping for safety."));
-		    		)
-		    		break;
-			}
-		}
+                        // Safety break, so we don't read past
+                        // the end of the  movie.
+                        log_swferror(_("Hit stream-end tag, "
+                            "but not at the advertised SWF end; "
+                            "stopping for safety."));
+                        )
+                        break;
+                }
+            }
 
-		SWF::TagLoadersTable::loader_function lf = NULL;
+            SWF::TagLoadersTable::loader_function lf = NULL;
 
-		if (tag_type == SWF::SHOWFRAME)
-		{
-			// show frame tag -- advance to the next frame.
+            if (tag == SWF::SHOWFRAME)
+            {
+                // show frame tag -- advance to the next frame.
 
-			IF_VERBOSE_PARSE(
-				log_parse("  show_frame");
-			);
+                IF_VERBOSE_PARSE(
+                    log_parse("  show_frame");
+                );
 
-			size_t floaded = incrementLoadedFrames();
-			if ( floaded == m_frame_count )
-			{
-				str.close_tag();
-				tag_type = str.open_tag();
-				if ( tag_type != SWF::END )
-				{
-					IF_VERBOSE_MALFORMED_SWF(
-					log_swferror(_("last expected SHOWFRAME "
-						"in SWF stream '%s' isn't "
-						"followed by an END (%d)."),
-						get_url(), tag_type);
-					);
-				}
-				goto parse_tag;
-			}
-
-		}
-		else if (_tag_loaders.get(tag_type, &lf))
+                size_t floaded = incrementLoadedFrames();
+                if (floaded == m_frame_count)
                 {
-			// call the tag loader.  The tag loader should add
-			// characters or tags to the movie data structure.
-			(*lf)(str, tag_type, *this);
-		}
-		else
-		{
-			// no tag loader for this tag type.
-            log_error(_("*** no tag loader for type %d (movie)"), tag_type);
-            IF_VERBOSE_PARSE(
-                std::ostringstream ss;
-                dumpTagBytes(str, ss);
-                log_error("tag dump follows: %s", ss.str());
-            );
-		}
+                    str.close_tag();
+                    tag = str.open_tag();
+                    if (tag != SWF::END )
+                    {
+                        IF_VERBOSE_MALFORMED_SWF(
+                        log_swferror(_("last expected SHOWFRAME "
+                            "in SWF stream '%s' isn't "
+                            "followed by an END (%d)."),
+                            get_url(), tag);
+                        );
+                    }
+                    goto parse_tag;
+                }
 
-		str.close_tag();
+            }
+            else if (_tag_loaders.get(tag, &lf)) {
+                // call the tag loader.  The tag loader should add
+                // characters or tags to the movie data structure.
+                (*lf)(str, tag, *this, _runInfo);
+            }
+            else {
+                // no tag loader for this tag type.
+                log_error(_("*** no tag loader for type %d (movie)"), tag);
+                IF_VERBOSE_PARSE(
+                    std::ostringstream ss;
+                    dumpTagBytes(str, ss);
+                    log_error("tag dump follows: %s", ss.str());
+                );
+            }
 
-		setBytesLoaded(str.tell());
-	}
+            str.close_tag();
 
-	} catch (const std::exception& e) {
+            setBytesLoaded(str.tell());
+        }
+
+	} catch (const ParserException& e) {
 		// FIXME: we should be setting some variable
 		//        so that it is possible for clients
 		//        to check the parser status
@@ -704,14 +636,20 @@ parse_tag:
 		//        and make sure any wait_for_frame call is
 		//        released (condition set and false result)
 		log_error(_("Parsing exception: %s"), e.what());
+
 	}
 
 	// Make sure we won't leave any pending writers
 	// on any eventual fd-based IOChannel.
 	str.consumeInput();
+    
+    // Set bytesLoaded to the current stream position unless it's greater
+    // than the reported length. TODO: should we be trying to continue
+    // parsing after an exception?
+    setBytesLoaded(std::min<size_t>(str.tell(), _swf_end_pos));
 
 	size_t floaded = get_loading_frame();
-	if ( ! m_playlist[floaded].empty() )
+	if (!m_playlist[floaded].empty())
 	{
 		IF_VERBOSE_MALFORMED_SWF(
 		log_swferror(_("%d control tags are NOT followed by"
@@ -722,8 +660,9 @@ parse_tag:
 	if ( m_frame_count > floaded )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
-		log_swferror(_("%d frames advertised in header, but only %d SHOWFRAME tags "
-			"found in stream. Pretending we loaded all advertised frames"), m_frame_count, floaded);
+		log_swferror(_("%d frames advertised in header, but only %d "
+                "SHOWFRAME tags found in stream. Pretending we loaded "
+                "all advertised frames"), m_frame_count, floaded);
 		);
 		boost::mutex::scoped_lock lock(_frames_loaded_mutex);
 		_frames_loaded = m_frame_count;
@@ -754,17 +693,15 @@ SWFMovieDefinition::incrementLoadedFrames()
 				"the advertised number in header (%d)."),
 				get_url(), _frames_loaded,
 				m_frame_count);
-		);
-		//m_playlist.resize(_frames_loaded+1);
+		)
 	}
 
 #ifdef DEBUG_FRAMES_LOAD
-	log_debug(_("Loaded frame %u/%u"),
-		_frames_loaded, m_frame_count);
+	log_debug(_("Loaded frame %u/%u"), _frames_loaded, m_frame_count);
 #endif
 
 	// signal load of frame if anyone requested it
-	// FIXME: _waiting_form_frame needs mutex ?
+	// FIXME: _waiting_for_frame needs mutex ?
 	if (_waiting_for_frame && _frames_loaded >= _waiting_for_frame )
 	{
 		// or should we notify_one ?
@@ -776,7 +713,8 @@ SWFMovieDefinition::incrementLoadedFrames()
 }
 
 void
-SWFMovieDefinition::export_resource(const std::string& symbol, resource* res)
+SWFMovieDefinition::export_resource(const std::string& symbol,
+        ExportableResource* res)
 {
 	// _exportedResources access should be protected by a mutex
 	boost::mutex::scoped_lock lock(_exportedResourcesMutex);
@@ -786,11 +724,12 @@ SWFMovieDefinition::export_resource(const std::string& symbol, resource* res)
 }
 
 
-boost::intrusive_ptr<resource>
-SWFMovieDefinition::get_exported_resource(const std::string& symbol)
+boost::intrusive_ptr<ExportableResource>
+SWFMovieDefinition::get_exported_resource(const std::string& symbol) const
 {
 #ifdef DEBUG_EXPORTS
-	log_debug(_("get_exported_resource called, frame count=%u"), m_frame_count);
+	log_debug("get_exported_resource(%s) called, loading frame:%u",
+            symbol, m_frame_count);
 #endif
 
 	// Don't call get_exported_resource() from this movie loader
@@ -831,8 +770,14 @@ SWFMovieDefinition::get_exported_resource(const std::string& symbol)
 		// _exportedResources access is thread-safe
 		{
 			boost::mutex::scoped_lock lock(_exportedResourcesMutex);
-			ExportMap::iterator it = _exportedResources.find(symbol);
-			if ( it != _exportedResources.end() ) return it->second;
+			ExportMap::const_iterator it = _exportedResources.find(symbol);
+			if ( it != _exportedResources.end() )
+            {
+#ifdef DEBUG_EXPORTS
+	            log_debug(" resource found, loading frame:%u", new_loading_frame);
+#endif
+                return it->second;
+            }
 		}
 
         // We checked last (or past-last) advertised frame. 
@@ -892,9 +837,10 @@ SWFMovieDefinition::get_exported_resource(const std::string& symbol)
 		log_error("No export symbol %s found in movie %s. "
 			"Frames loaded %d/%d",
 			symbol, _url, loading_frame, m_frame_count);
+        //abort();
 	}
 
-	return boost::intrusive_ptr<resource>(0); // 0
+	return boost::intrusive_ptr<ExportableResource>(0); // 0
 
 }
 
@@ -926,16 +872,12 @@ SWFMovieDefinition::markReachableResources() const
 		i->second->setReachable();
 	}
 
-	for (BitmapMap::const_iterator i=m_bitmap_characters.begin(), e=m_bitmap_characters.end(); i!=e; ++i)
+	for (Bitmaps::const_iterator i = _bitmaps.begin(), e = _bitmaps.end();
+            i != e; ++i)
 	{
 		i->second->setReachable();
 	}
 
-	for (BitmapVect::const_iterator i=m_bitmap_list.begin(), e=m_bitmap_list.end(); i!=e; ++i)
-	{
-		(*i)->setReachable();
-	}
-
 	for (SoundSampleMap::const_iterator i=m_sound_samples.begin(), e=m_sound_samples.end(); i!=e; ++i)
 	{
 		i->second->setReachable();
@@ -944,13 +886,15 @@ SWFMovieDefinition::markReachableResources() const
 	// TODO: turn this into a markExportedResources()
 	{
 		boost::mutex::scoped_lock lock(_exportedResourcesMutex);
-		for (ExportMap::const_iterator i=_exportedResources.begin(), e=_exportedResources.end(); i!=e; ++i)
+		for (ExportMap::const_iterator i=_exportedResources.begin(),
+                e=_exportedResources.end(); i!=e; ++i)
 		{
 			i->second->setReachable();
 		}
 	}
 
-	for (ImportVect::const_iterator i=m_import_source_movies.begin(), e=m_import_source_movies.end(); i!=e; ++i)
+	for (ImportVect::const_iterator i=m_import_source_movies.begin(),
+            e=m_import_source_movies.end(); i!=e; ++i)
 	{
 		(*i)->setReachable();
 	}
@@ -962,7 +906,8 @@ SWFMovieDefinition::markReachableResources() const
 #endif // GNASH_USE_GC
 
 void
-SWFMovieDefinition::importResources(boost::intrusive_ptr<movie_definition> source, Imports& imports)
+SWFMovieDefinition::importResources(
+        boost::intrusive_ptr<movie_definition> source, Imports& imports)
 {
 	size_t importedSyms=0;
 	for (Imports::iterator i=imports.begin(), e=imports.end(); i!=e; ++i)
@@ -970,30 +915,33 @@ SWFMovieDefinition::importResources(boost::intrusive_ptr<movie_definition> sourc
 		int id = i->first;
 		const std::string& symbolName = i->second;
 
-	        boost::intrusive_ptr<resource> res = source->get_exported_resource(symbolName);
-	        if (!res)
-	        {
-			log_error(_("import error: could not find resource '%s' in movie '%s'"),
-				symbolName, source->get_url());
+        boost::intrusive_ptr<ExportableResource> res =
+            source->get_exported_resource(symbolName);
+
+        if (!res)
+        {
+			log_error(_("import error: could not find resource '%s' in "
+                        "movie '%s'"), symbolName, source->get_url());
 			continue;
-	        }
-	        else if (font* f = res->cast_to_font())
+        }
+        else if (Font* f = dynamic_cast<Font*>(res.get()))
 		{
 			// Add this shared font to the currently-loading movie.
 			add_font(id, f);
 			++importedSyms;
-	        }
-	        else if (character_def* ch = res->cast_to_character_def())
-	        {
-			// Add this character to the loading movie.
-			add_character(id, ch);
-			++importedSyms;
-	        }
-	        else
-	        {
-			log_error(_("importResources error: unsupported import of '%s' from movie '%s' has unknown type"),
-				symbolName, source->get_url());
-	        }
+        }
+        else if (character_def* ch = dynamic_cast<character_def*>(res.get()))
+        {
+            // Add this character to the loading movie.
+            add_character(id, ch);
+            ++importedSyms;
+        }
+        else
+        {
+            log_error(_("importResources error: unsupported import of '%s' "
+                "from movie '%s' has unknown type"),
+                symbolName, source->get_url());
+        }
 	}
 
 	if ( importedSyms )
