@@ -1,6 +1,6 @@
 // AudioDecoderSimple.h: Audio decoding using "simple" internal decoders.
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,14 +17,18 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
-#ifndef __AUDIODECODERSIMPLE_H__
-#define __AUDIODECODERSIMPLE_H__
+#ifndef GNASH_AUDIODECODERSIMPLE_H
+#define GNASH_AUDIODECODERSIMPLE_H
 
-#include "log.h"
-#include "AudioDecoder.h"
+#include "AudioDecoder.h" // for inheritance
+#include "MediaParser.h" // for audioCodecType enum (composition)
 
+// Forward declarations
 namespace gnash {
-	class SoundInfo;
+    namespace media {
+        class SoundInfo;
+        class AudioInfo;
+    }
 }
 
 namespace gnash {
@@ -34,17 +38,38 @@ namespace media {
 class AudioDecoderSimple : public AudioDecoder {
 
 public:
-	AudioDecoderSimple();
+
+	/// @param info
+	/// 	AudioInfo class with all the info needed to decode
+	///     the sound correctly. Throws a MediaException on fatal
+	///     error.
+    ///
+    /// @throws MediaException on failure
+    ///
+	AudioDecoderSimple(const AudioInfo& info);
+	
+	/// @param info
+	/// 	SoundInfo class with all the info needed to decode
+	///     the sound correctly. Throws a MediaException on fatal
+	///     error.	
+    ///
+    /// @throws MediaException on failure
+    ///
+	AudioDecoderSimple(const SoundInfo& info);
+
 	~AudioDecoderSimple();
 
-	bool setup(AudioInfo* info);
-
-	bool setup(SoundInfo* info);
-
-	boost::uint8_t* decode(boost::uint8_t* input, boost::uint32_t inputSize, boost::uint32_t& outputSize, boost::uint32_t& decodedBytes, bool parse);
+    // See dox in AudioDecoder.h
+	boost::uint8_t* decode(const boost::uint8_t* input, boost::uint32_t inputSize, boost::uint32_t& outputSize, boost::uint32_t& decodedBytes, bool parse);
 
 private:
 
+    // throws MediaException on failure
+	void setup(const AudioInfo& info);
+
+    // throws MediaException on failure
+	void setup(const SoundInfo& info);
+
 	// codec
 	audioCodecType _codec;
 
