@@ -1,6 +1,6 @@
 // MediaHandlerFfmpeg.h: FFMPEG media handler, for Gnash
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,21 +31,38 @@
 namespace gnash {
 namespace media {
 
+/// FFMPEG-based media handler module
+//
+/// The module implements the MediaHandler factory as required
+/// by Gnash core for a loadable media handler module.
+///
+/// It uses libavformat and libavcodec:
+/// http://www.irisa.fr/texmex/people/dufouil/ffmpegdoxy/index.html
+///
+/// Starting point is MediaHandlerFfmpeg.
+/// 
+namespace ffmpeg {
+
+/// FFMPEG based MediaHandler
 class DSOEXPORT MediaHandlerFfmpeg : public MediaHandler
 {
 public:
 
 	virtual std::auto_ptr<MediaParser> createMediaParser(std::auto_ptr<IOChannel> stream);
 
-	virtual std::auto_ptr<VideoDecoder> createVideoDecoder(VideoInfo& info);
+	virtual std::auto_ptr<VideoDecoder> createVideoDecoder(const VideoInfo& info);
+	
+	virtual std::auto_ptr<VideoConverter>
+		createVideoConverter(ImgBuf::Type4CC srcFormat, ImgBuf::Type4CC dstFormat);
 
-	virtual std::auto_ptr<AudioDecoder> createAudioDecoder(AudioInfo& info);
+	virtual std::auto_ptr<AudioDecoder> createAudioDecoder(const AudioInfo& info);
 
     virtual size_t getInputPaddingSize() const;
 
 };
 
 
+} // gnash.media.ffmpeg namespace 
 } // gnash.media namespace 
 } // namespace gnash
 
