@@ -1,6 +1,6 @@
 // AudioDecoderGst.h: Audio decoding using Gstreamer.
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,23 +28,32 @@
 #include "AudioDecoder.h"
 
 #include <gst/gst.h>
-#include "image.h"
+#include "GnashImage.h"
 
 #include "swfdec_codec_gst.h"
 
+// Forward declarations
+namespace gnash {
+    namespace media {
+        class AudioInfo;
+        class SoundInfo;
+    }
+}
+
 namespace gnash {
 namespace media {
+namespace gst {
 
-/// Video decoding using Gstreamer.
+/// GST based AudioDecoder
 class DSOEXPORT AudioDecoderGst : public AudioDecoder {
 	
 public:
-    AudioDecoderGst(AudioInfo& info);
+    AudioDecoderGst(const AudioInfo& info);
     AudioDecoderGst(SoundInfo& info);
 
     ~AudioDecoderGst();
 
-    boost::uint8_t* decode(boost::uint8_t* input, boost::uint32_t inputSize,
+    boost::uint8_t* decode(const boost::uint8_t* input, boost::uint32_t inputSize,
                            boost::uint32_t& outputSize, boost::uint32_t& decodedData,
                            bool /*parse*/);
     boost::uint8_t* decode(const EncodedAudioFrame& ef, boost::uint32_t& outputSize);
@@ -59,6 +68,7 @@ private:
 
 };
 
+} // gnash.media.gst namespace
 } // media namespace
 } // gnash namespace
 
