@@ -1,6 +1,6 @@
 // GstUtil.cpp: Generalized Gstreamer utilities for pipeline configuration.
 //
-//   Copyright (C) 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 
 namespace gnash {
 namespace media {
+namespace gst {
 
 GstElement* GstUtil::get_audiosink_element()
 {   
@@ -143,7 +144,7 @@ GstUtil::check_missing_plugins(GstCaps* caps)
     return false;
 }
 
-
+} // gnash.media.gst namespace
 } // gnash.media namespace 
 } // namespace gnash
 
