@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ namespace gnash
 
 class CQue {
 public:
-    typedef std::deque<amf::Buffer *> que_t;
+    typedef std::deque<boost::shared_ptr<amf::Buffer> > que_t;
 #ifdef USE_STATS_QUEUE
     typedef struct {
 	struct timespec start;
@@ -48,12 +48,12 @@ public:
     CQue(const std::string &str) { _name = str; };
     ~CQue();
     // Push data onto the que
-    bool push(gnash::Network::byte_t *data, int nbytes);
-    bool push(amf::Buffer *data);
+    bool push(boost::uint8_t *data, int nbytes);
+    bool push(boost::shared_ptr<amf::Buffer> data);
     // Pop the first date element off the que
-    amf::Buffer *pop();
+    boost::shared_ptr<amf::Buffer> DSOEXPORT pop();
     // Peek at the first date element witjhout removing it from the que
-    amf::Buffer *peek();
+    boost::shared_ptr<amf::Buffer> peek();
     // Get the number of elements in the que
     size_t DSOEXPORT size();
     // Wait for a condition variable to trigger
@@ -63,13 +63,15 @@ public:
     // Empty the que of all data. 
     void clear();
     // Remove a range of elements
-    void remove(amf::Buffer *begin, amf::Buffer *end);
+    void remove(boost::shared_ptr<amf::Buffer> begin, boost::shared_ptr<amf::Buffer> end);
 //     // Remove an element
-//    void remove(amf::Buffer *it);
-    void remove(amf::Buffer *it);
+//    void remove(boost::shared_ptr<amf::Buffer> it);
+    void remove(boost::shared_ptr<amf::Buffer> it);
     // Merge sucessive buffers into one single larger buffer. This is for some
     // protocols, than have very long headers.
-    amf::Buffer *merge(amf::Buffer *begin);
+    boost::shared_ptr<amf::Buffer> merge(boost::shared_ptr<amf::Buffer> begin);
+
+    boost::shared_ptr<amf::Buffer> operator[] (int index) { return _que[index]; };
     
     // Dump the data to the terminal
     void dump();
@@ -80,16 +82,16 @@ public:
     const std::string &getName() { return _name; }
 private:
     // an optional name for the queue, only used for debugging messages to make them unique
-    std::string _name;
+    std::string		_name;
     // The queue itself
-    que_t _que;
+    que_t		_que;
 
     // A condition variable used to signal the other thread when the que has data
     boost::condition	_cond;
     // This is the mutex used by the condition variable. It needs to be separate from the
     // one used to lock access to the que.
     boost::mutex	_cond_mutex;
-    // This is the mutex that control access to the que.
+    // This is the mutex that controls access to the que.
     boost::mutex	_mutex;
 #ifdef USE_STATS_QUEUE
     que_stats_t		_stats;
