@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,13 @@
 #include "movie_definition.h"
 #include "movie_instance.h"
 #include "movie_root.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "as_environment.h"
 #include "gnash.h" // for create_movie and create_library_movie and for gnash::key namespace
 #include "VM.h" // for initialization
 #include "sound_handler.h" // for creating the "test" sound handlers
-#include "render.h" // for get_render_handler
-#include "types.h" // for rgba class
+#include "NullSoundHandler.h"
+#include "RGBA.h" // for rgba class
 #include "FuzzyPixel.h"
 #include "render.h"
 #include "render_handler.h"
@@ -41,10 +41,17 @@
 #include "render_handler_cairo.h"
 #endif
 
+#include "MediaHandler.h"
+#ifdef USE_FFMPEG
+# include "ffmpeg/MediaHandlerFfmpeg.h"
+#elif defined(USE_GST)
+# include "gst/MediaHandlerGst.h"
+#endif
+
 #include <cstdio>
 #include <string>
 #include <memory> // for auto_ptr
-#include <cmath> // for ceil and exp2
+#include <cmath> // for ceil and (possibly) exp2
 #include <iostream>
 
 #define SHOW_INVALIDATED_BOUNDS_ON_ADVANCE 1
@@ -67,16 +74,24 @@ MovieTester::MovieTester(const std::string& url)
 	// Initialize gnash code lib
 	gnashInit();
 
-	// Set base url *before* calling create_movie
-	// TODO: use PWD if url == '-'
-	set_base_url(url);
+	// Initialize the testing media handlers
+	initTestingMediaHandlers();
+
+	// Initialize the sound handler(s)
+	initTestingSoundHandlers();
+
+    _runInfo.reset(new RunInfo(url));
+    _runInfo->setSoundHandler(_sound_handler);
+
+    _runInfo->setStreamProvider(boost::shared_ptr<StreamProvider>(
+                new StreamProvider));
 
 	if ( url == "-" )
 	{
-		std::auto_ptr<tu_file> in (
+		std::auto_ptr<IOChannel> in (
 				noseek_fd_adapter::make_stream(fileno(stdin))
 				);
-		_movie_def = gnash::create_movie(in, url, false);
+		_movie_def = gnash::create_movie(in, url, *_runInfo, false);
 	}
 	else
 	{
@@ -96,7 +111,8 @@ MovieTester::MovieTester(const std::string& url)
 #endif
 		}
 		// _url should be always set at this point...
-		_movie_def = gnash::create_library_movie(urlObj, NULL, false);
+		_movie_def = gnash::create_library_movie(urlObj, *_runInfo,
+                NULL, false);
 	}
 
 	if ( ! _movie_def )
@@ -104,10 +120,7 @@ MovieTester::MovieTester(const std::string& url)
 		throw GnashException("Could not load movie from "+url);
 	}
 
-	// Initialize the sound handler(s)
-	initTestingSoundHandlers();
-
-	_movie_root = &(VM::init(*_movie_def, _clock).getRoot());
+	_movie_root = new movie_root(*_movie_def, _clock, *_runInfo);
 
 	// Initialize viewport size with the one advertised in the header
 	_width = unsigned(_movie_def->get_width_pixels());
@@ -224,6 +237,31 @@ void
 MovieTester::advanceClock(unsigned long ms)
 {
 	_clock.advance(ms);
+
+    // We need to fetch as many samples
+    // as needed for a theoretical 44100hz loop.
+    // That is 44100 samples each second.
+    // 44100/1000 = x/ms
+    //  x = (44100*ms) / 1000
+    unsigned int nSamples = (441*ms) / 10;
+    if ( ms%10 )
+    {
+        log_error("MovieTester::advanceClock: %d ms lost in sound advancement",
+            ms%10);
+    }
+
+    unsigned int toFetch = nSamples*2;
+
+    log_debug("advanceClock(%d) needs to fetch %d samples", ms, toFetch);
+
+    boost::int16_t samples[1024];
+    while (toFetch)
+    {
+        unsigned int n = std::min(toFetch, 1024u);
+        _sound_handler->fetchSamples((boost::int16_t*)&samples, n);
+        toFetch -= n;
+    }
+    
 }
 
 void
@@ -273,7 +311,7 @@ MovieTester::resizeStage(int x, int y)
 }
 
 const character*
-MovieTester::findDisplayItemByName(const sprite_instance& mc,
+MovieTester::findDisplayItemByName(const MovieClip& mc,
 		const std::string& name) 
 {
 	const DisplayList& dlist = mc.getDisplayList();
@@ -281,7 +319,7 @@ MovieTester::findDisplayItemByName(const sprite_instance& mc,
 }
 
 const character*
-MovieTester::findDisplayItemByDepth(const sprite_instance& mc,
+MovieTester::findDisplayItemByDepth(const MovieClip& mc,
 		int depth)
 {
 	const DisplayList& dlist = mc.getDisplayList();
@@ -529,11 +567,6 @@ MovieTester::canTestVideo() const
 {
 	if ( ! canTestSound() ) return false;
 
-#ifdef USE_MAD
-	// mad doesn't support video !
-	return false;
-#endif
-
 	return true;
 }
 
@@ -541,19 +574,25 @@ void
 MovieTester::initTestingSoundHandlers()
 {
 
-#ifdef SOUND_SDL
-	std::cout << "Creating SDL sound handler" << std::endl;
-        _sound_handler.reset( gnash::media::create_sound_handler_sdl() );
-#elif defined(SOUND_GST)
-	std::cout << "Creating GST sound handler" << std::endl;
-        _sound_handler.reset( gnash::media::create_sound_handler_gst() );
+    _sound_handler.reset( new sound::NullSoundHandler() );
+}
+
+void
+MovieTester::initTestingMediaHandlers()
+{
+
+	std::auto_ptr<media::MediaHandler> handler;
+
+#ifdef USE_FFMPEG
+	handler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
+#elif defined(USE_GST)
+        handler.reset( new gnash::media::gst::MediaHandlerGst() );
 #else
 	std::cerr << "Neigher SOUND_SDL nor SOUND_GST defined" << std::endl;
 	return;
-	//exit(1);
 #endif
 
-	gnash::set_sound_handler(_sound_handler.get());
+	gnash::media::MediaHandler::set(handler);
 }
 
 void
