@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,10 @@
 #include "Range2d.h"
 #include "GnashKey.h"
 #include "sound_handler.h" // for creating the "test" sound handlers
-#include "types.h" // for rgba class
 #include "render_handler.h" // for dtor visibility by auto_ptr
 #include "movie_instance.h" 
 #include "ManualClock.h" // for composition
+#include "RunInfo.h" // For initialization.
 
 #include <memory> // for auto_ptr
 #include <string> 
@@ -56,10 +56,11 @@
 namespace gnash {
 	class movie_definition;
 	class movie_root;
-	class sprite_instance;
+	class MovieClip;
 	class character;
 	class FuzzyPixel;
 	class VirtualClock;
+	class rgba;
 }
 
 namespace gnash {
@@ -142,7 +143,7 @@ public:
 	/// Return NULL if there's no character with that name in
 	/// the sprite's display list.
 	///
-	const character* findDisplayItemByName(const sprite_instance& mc,
+	const character* findDisplayItemByName(const MovieClip& mc,
 			const std::string& name);
 
 	/// Find a character in the display list of a sprite by depth.
@@ -150,11 +151,11 @@ public:
 	/// Return NULL if there's no character at that depth in
 	/// the sprite's display list.
 	///
-	const character* findDisplayItemByDepth(const sprite_instance& mc,
+	const character* findDisplayItemByDepth(const MovieClip& mc,
 			int depth);
 
 	/// Get the topmost sprite instance of this movie
-	gnash::sprite_instance* getRootMovie() {
+	gnash::MovieClip* getRootMovie() {
 		return _movie;
 	}
 
@@ -318,8 +319,9 @@ private:
 
 	gnash::movie_instance* _movie;
 
-	std::auto_ptr<media::sound_handler> _sound_handler;
+    boost::shared_ptr<sound::sound_handler> _sound_handler;
 
+    std::auto_ptr<RunInfo> _runInfo;
 	/// Current pointer position - X ordinate
 	int _x;
 
