@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@
 #define INPUT_FILENAME "DefineEditTextVariableNameTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "character.h"
-#include "dlist.h"
+#include "DisplayList.h"
 #include "log.h"
 
 #include "check.h"
@@ -39,11 +39,11 @@ main(int /*argc*/, char** /*argv*/)
 	string filename = INPUT_FILENAME;
 	MovieTester tester(filename);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	check_equals(root->get_frame_count(), 14);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->get_play_state(), MovieClip::PLAY);
 	check_equals(root->get_current_frame(), 0);
 
 	const character* mc1 = tester.findDisplayItemByName(*root, "mc1");
@@ -59,12 +59,12 @@ main(int /*argc*/, char** /*argv*/)
 	for (unsigned f=root->get_current_frame(); f<root->get_frame_count()-1; ++f)
 	{
 		check_equals(root->get_current_frame(), f);
-		check_equals(root->get_play_state(), sprite_instance::PLAY);
+		check_equals(root->get_play_state(), MovieClip::PLAY);
 		tester.advance();
 	}
 
 	// does stop() on last frame
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->get_play_state(), MovieClip::STOP);
 	check_equals(root->get_current_frame(), root->get_frame_count()-1);
 
 }
