@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@
 #include "gnashconfig.h"
 #endif
 
-#include <stdarg.h>
+#include "GnashSystemIOHeaders.h" // write()
+#include <cstdarg>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <cerrno>
 
 extern "C"{
-#include <unistd.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -234,7 +234,7 @@ dump_shm(bool convert, bool out)
 	     << ", Size is: " << dec << size << endl;
     }
     
-    LcShm lc;
+    amf::LcShm lc;
     lc.connect(key);
     lc.dump();
     
@@ -354,7 +354,6 @@ void dump_ctrl(void *inmem)
     cerr << _("\tTotal # of bytes: ") << ptr->getSize() << endl;
 }
 
-
 // Local Variables:
 // mode: C++
 // indent-tabs-mode: t
